import xmlrpc.client

import requests


class RequestsTransport(xmlrpc.client.Transport):
	def request(self, host, handler, data, verbose=False):
		# set the headers, including the user-agent
		headers = {
			"User-Agent": "my-user-agent",
			"Content-Type": "text/xml",
			"Accept-Encoding": "gzip"
		}
		url = "https://%s%s" % (host, handler)
		response = None
		try:
			response = requests.post(url, data=data, headers=headers)
			response.raise_for_status()
			return self.parse_response(response)
		except requests.RequestException as e:
			if response is None:
				raise xmlrpc.client.ProtocolError(url, 500, str(e), "")
			else:
				raise xmlrpc.client.ProtocolError(url, response.status_code, str(e), response.headers)

	def parse_response(self, resp):
		"""
		Parse the xmlrpc response.
		"""
		p, u = self.getparser()
		p.feed(resp.text)
		p.close()
		return u.close()


###############################################################
# Name:			 Editor.py
# Purpose:	 Subclass of StyledTextControl providing songpress
#            editor: loading, saving and syntax hilighting
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2009-01-16
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

import codecs

from wx.stc import *
import wx.lib, wx.lib.newevent

from .SDIMainFrame import *
from .SongTokenizer import *
from . import Transpose


EventTextChanged, EVT_TEXT_CHANGED = wx.lib.newevent.NewEvent()

def get_text_from_clipboard():
	"""
	Return text in clipboard, or None
	"""
	if wx.TheClipboard.IsOpened():
		return None
	wx.TheClipboard.Open()
	do = wx.TextDataObject()
	if not wx.TheClipboard.GetData(do):
		return None
	wx.TheClipboard.Close()
	return do.GetText()

class Editor(StyledTextCtrl):

	def __init__(self, spframe, interactive=True, frame=None):
		self.frame = spframe.frame if frame is None else frame
		StyledTextCtrl.__init__(self, self.frame)
		self.spframe = spframe
		self.interactive = interactive
		if self.interactive:
			self.lastPos = 0
			self.frame.Bind(EVT_STC_CHANGE, self.OnTextChange, self)
			self.frame.Bind(EVT_STC_UPDATEUI, self.OnUpdateUI, self)
			self.Bind(EVT_STC_DOUBLECLICK, self.OnDoubleClick, self)
			self.Bind(wx.EVT_CHAR, self.OnChar, self)
			self.autoChangeMode = False
		self.frame.Bind(EVT_STC_STYLENEEDED, self.OnStyleNeeded, self)
		self.STC_STYLE_NORMAL = 11
		self.STC_STYLE_CHORD = 12
		self.STC_STYLE_COMMAND = 13
		self.STC_STYLE_ATTR = 14
		self.STC_STYLE_CHORUS = 15
		self.STC_STYLE_COMMENT = 16
		self.SetFont("Lucida Console", 12)
		self.SetLexer(STC_LEX_CONTAINER)
		self.StyleSetForeground(self.STC_STYLE_NORMAL, wx.Colour(0, 0, 0))
		self.StyleSetForeground(self.STC_STYLE_CHORUS, wx.Colour(0, 0, 0))
		self.StyleSetForeground(self.STC_STYLE_CHORD, wx.Colour(255, 0, 0))
		self.StyleSetForeground(self.STC_STYLE_COMMAND, wx.Colour(0, 0, 255))
		self.StyleSetForeground(self.STC_STYLE_ATTR, wx.Colour(0, 128, 0))
		self.StyleSetForeground(self.STC_STYLE_COMMENT, wx.Colour(128, 128, 128))
		self.StyleSetBold(self.STC_STYLE_CHORUS, True)
		#Dummy "token": we artificially replace every normalToken into a chorusToken when we are
		#inside chorus.  Then, we can associate the chorus style in self.tokenStyle dictionary.
		self.chorusToken = 'chorusToken'
		self.tokenStyle = {
			SongTokenizer.openCurlyToken: self.STC_STYLE_COMMAND,
			SongTokenizer.closeCurlyToken: self.STC_STYLE_COMMAND,
			SongTokenizer.normalToken: self.STC_STYLE_NORMAL,
			SongTokenizer.commandToken: self.STC_STYLE_COMMAND,
			SongTokenizer.attrToken: self.STC_STYLE_ATTR,
			SongTokenizer.chordToken: self.STC_STYLE_CHORD,
			SongTokenizer.closeChordToken: self.STC_STYLE_CHORD,
			SongTokenizer.colonToken: self.STC_STYLE_COMMAND,
			SongTokenizer.commentToken: self.STC_STYLE_COMMENT,
			self.chorusToken: self.STC_STYLE_CHORUS
		}
		#self.chorus[i] == True iff, at the end of line i, we are still in chorus (i.e. bold) mode
		self.chorus = []

	def SetFont(self, face, size):
		font = wx.Font(
			size,
			wx.FONTFAMILY_DEFAULT,
			wx.FONTSTYLE_NORMAL,
			wx.FONTWEIGHT_NORMAL,
			faceName = face
		)
		self.StyleSetFont(STC_STYLE_DEFAULT, font)
		#I don't know why, but the following line is necessary in order to make
		#the font bold
		self.StyleSetFont(self.STC_STYLE_CHORUS, font)

	def New(self):
		self.ClearAll()

	def Open(self):
		self.ClearAll()
		self.LoadFile(self.spframe.document)

	def Save(self):
		#self.SaveFile(self.spframe.document)
		t = self.GetText()
		f = codecs.open(self.spframe.document, 'w', 'utf-8')
		f.write(t)
		f.close()
		

	def GetTextOrSelection(self):
		start, end = self.GetSelection()
		if start == end:
			return self.GetText()
		return self.GetSelectedText()

	def ReplaceTextOrSelection(self, text):
		start, end = self.GetSelection()
		if start == end:
			self.SetText(text)
		else:
			self.ReplaceSelection(text)

	def GetChordUnderCursor(self):
		"""Return info about chord under cursor, or None

		Return a 3-tuple: (begin, end, chord)
			begin: position before open bracket
			end: position after close bracket
			chord: chord, without brackets
		"""
		pos, dummy = self.GetSelection()
		char = ""
		if self.GetSelectedText().find('[') != -1:
			start = pos
			while char != '[':
				next = self.PositionAfter(start)
				char = self.GetTextRange(start, next)
				start = next
			start = self.PositionBefore(start)
		else:
			start = self.PositionBefore(pos)
			while start >= 0 and char != '[' and char != '\n' and char != ']':
				char = self.GetTextRange(start, self.PositionAfter(start))
				if start == 0:
					start = -1
				else:
					start = self.PositionBefore(start)
			if start >= 0:
				start = self.PositionAfter(start)
			else:
				start = 0
		if char == '[':
			end = self.PositionBefore(pos)
			l = self.GetLength()
			while end < l and char != ']' and char != '\n':
				char = self.GetTextRange(end, self.PositionAfter(end))
				end = self.PositionAfter(end)
			if char == ']':
				return (start, end, self.GetTextRange(self.PositionAfter(start), self.PositionBefore(end)))
		return None

	def SelectNextChord(self):
		dummy, pos = self.GetSelection()
		n = self.GetLength()
		c = ''
		while pos < n:
			while pos < n and c != '[':
				c = self.GetTextRange(pos, self.PositionAfter(pos))
				pos = self.PositionAfter(pos)
			if c == '[':
				start = pos
				while pos < n and c != ']' and c != '\n':
					c = self.GetTextRange(pos, self.PositionAfter(pos))
					pos = self.PositionAfter(pos)
				if c == ']':
					self.SetSelection(self.PositionBefore(start), pos)
					return True
		return False

	def SelectPreviousChord(self):
		pos, dummy = self.GetSelection()
		c = ''
		done = False
		while not done:
			while not done and c != ']':
				c = self.GetTextRange(pos, self.PositionAfter(pos))
				if pos <= 0:
					done = True
				else:
					pos = self.PositionBefore(pos)
			if c == ']':
				end = pos
				while not done and c != '[' and c != '\n':
					c = self.GetTextRange(pos, self.PositionAfter(pos))
					if pos <= 0:
						pos = -1
						done = True
					else:
						pos = self.PositionBefore(pos)
				if pos == -1:
					pos = 0
				else:
					pos = self.PositionAfter(pos)
				if c == '[':
					self.SetSelection(pos, self.PositionAfter(self.PositionAfter(end)))
					return True
		return False

	def RemoveChordsInSelection(self):
		self.ReplaceSelection(Transpose.removeChords(self.GetSelectedText()))
		
	def CopyOnlyText(self):
		text = Transpose.removeChordPro(self.GetSelectedText())
		c = wx.TheClipboard
		if c.Open():
			c.SetData(wx.TextDataObject(text))
			c.Close()
		

	def OnTextChange(self, evt):
		if self.interactive:
			self.spframe.SetModified()
			self.spframe.TextUpdated()
			if not self.autoChangeMode:
				currentPos = self.GetCurrentPos()
				if currentPos - self.lastPos > 2:
					evt = EventTextChanged(lastPos=self.lastPos, currentPos=currentPos)
					wx.PostEvent(self.frame, evt)

	def OnUpdateUI(self, evt):
		self.lastPos = self.GetCurrentPos()
		evt.Skip(False)

	def OnDoubleClick(self, evt):
		t = self.GetChordUnderCursor()
		if t is not None:
			self.SetSelection(t[0], t[1])
		evt.Skip()

	def OnChar(self, evt):
		t = self.GetSelectedText()
		if t != '' and t[0] == '[' and t[-1] == ']':
			c = evt.GetKeyCode()
			if (c >= 65 and c <= 90) or (c >= 97 and c <= 122):
				s, e = self.GetSelection()
				self.SetSelection(self.PositionAfter(s), self.PositionBefore(e))
		evt.Skip()

	def AutoChangeMode(self, acm):
		self.autoChangeMode = acm
		
	def PasteChords(self):
		src = get_text_from_clipboard()
		if src is None:
			return
		self.BeginUndoAction()
		start, end = self.GetSelection()
		if start == end:
			l = self.LineFromPosition(start)
			end = self.PositionFromLine(l + len(src.splitlines()))
			if end == -1:
				end = self.GetLength()
			else:
				end = self.PositionBefore(end)
		prev = self.PositionBefore(end)
		while end > start and self.GetCharAt(prev) in [10, 13]:
			end = prev
			prev = self.PositionBefore(end)
		self.SetSelection(start, end)
		sel = self.GetSelectedText()
		if sel == "":
			sel = " "
		self.ReplaceSelection(Transpose.pasteChords(src, sel))
		self.EndUndoAction()

	def OnStyleNeeded(self, evt):
		end = evt.GetPosition()
		pos = self.GetEndStyled()
		ln = self.LineFromPosition(pos)
		start = self.PositionFromLine(ln)
		if ln > 0 and len(self.chorus) > ln-1:
			bold = self.chorus[ln-1]
		else:
			bold = False
		#changedBold is True iff self.chorus has changed for the current line, and thus we need
		#to process (at least) another line
		changedBold = False
		lc = self.GetLineCount()
		while (changedBold and ln < lc) or start < end:
			self.StartStyling(start)
			l = self.GetLine(ln)
			tkz = SongTokenizer(l)
			for tok in tkz:
				n = len(tok.content.encode('utf-8'))
				t = tok.token
				if bold and t == SongTokenizer.normalToken:
					t = self.chorusToken
				self.SetStyling(n, self.tokenStyle[t])
				if t == SongTokenizer.commandToken:
					if tok.content.upper() == 'SOC':
						bold = True
					elif tok.content.upper() == 'EOC':
						bold = False
			if len(self.chorus) > ln:
				changedBold = self.chorus[ln] != bold
				self.chorus[ln] = bold
			else:
				self.chorus.append(bold)
				changedBold = False
			ln = ln + 1
			start = self.PositionFromLine(ln)





# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Aug 25 2009)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
from . import i18n
i18n.register('songpress.NormalizeDialog')
# -*- coding: utf-8 -*- 


###########################################################################
## Class NormalizeDialog
###########################################################################

class NormalizeDialog ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = _(u"Cleanup Chords"), pos = wx.DefaultPosition, size = wx.Size( -1,-1 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		bSizer2 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.label1 = wx.StaticText( self, wx.ID_ANY, _(u"Notation:"), wx.DefaultPosition, wx.DefaultSize, 0 )
		self.label1.Wrap( -1 )
		bSizer2.Add( self.label1, 0, wx.ALL, 5 )
		
		fromNotationChoices = []
		self.fromNotation = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, fromNotationChoices, 0 )
		self.fromNotation.SetSelection( 0 )
		bSizer2.Add( self.fromNotation, 1, wx.ALL, 5 )
		
		
		bSizer1.Add( bSizer2, 1, wx.EXPAND, 5 )
		
		m_sdbSizer1 = wx.StdDialogButtonSizer()
		self.m_sdbSizer1OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer1.AddButton( self.m_sdbSizer1OK )
		self.m_sdbSizer1Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer1.AddButton( self.m_sdbSizer1Cancel )
		m_sdbSizer1.Realize();
		
		bSizer1.Add( m_sdbSizer1, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		bSizer1.Fit( self )
		
		# Connect Events
		self.fromNotation.Bind( wx.EVT_CHOICE, self.OnFromNotation )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnFromNotation( self, event ):
		event.Skip()
	




###############################################################
# Name:			 MyUpdateDialog.py
# Purpose:	 Dialog to select updates to apply
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2010-12-14
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

from xmlrpc.client import Server
import datetime
import platform
import traceback
import logging

import wx.lib.delayedresult as delayedresult
import wx

from .UpdateDialog import *
from .Globals import glb
from .MyUpdatePanel import MyUpdatePanel
from .proxiedxmlrpclib import RequestsTransport


from . import i18n
i18n.register('songpress.MyUpdateDialog')

def check_and_update(parent, preferences, force=False):
	"""
	Check for updates, and interact with user.
	
	Update mode is automatic if force=False, else manual.
	
	parent: parent window of any created modal dialogs
	"""
	def check_for_updates():
		"""
		Check and return available updates.
		
		Honour the frequency of update checking unless force is True.
		
		Return list of pair (version, description),
		where description is an html-formatted text.
		Raise an exception if anything goes wrong.
		"""
		if(
			force
			or preferences.updateLastCheck is None
			or datetime.datetime.now() > preferences.updateLastCheck + datetime.timedelta(days=preferences.updateFrequency)
		):
			tr = RequestsTransport()
			s = Server(preferences.updateUrl, transport=tr)
			# method returns a dictionary with those keys:
			# 'new_url': if defined, new url of the webservice
			# 'updates': list of 3-tuples (version, description, downloadUrl)
			u = s.checkForUpdates(
				glb.VERSION,
				preferences.updateFrequency,
				platform.system(),
				platform.architecture(),
				platform.platform(),
				platform.python_version(),
				wx.version(),
				i18n.getLang(),
			)
			u2 = [x for x in u['updates'] if x[0] not in preferences.ignoredUpdates]
			preferences.updateLastCheck = datetime.datetime.now()
			if 'new_url' in u:
				preferences.updateUrl = u['new_url']
			return u2
		return []

	def consume_updates(dr):
		try:
			u = dr.get()
			if len(u) > 0:
				d = MyUpdateDialog(parent, preferences, u)
				d.ShowModal()
			elif force:
				d = wx.MessageDialog(
					parent,
					_("Your version of Songpress is up to date."),
					_("Songpress"),
					wx.OK | wx.ICON_INFORMATION
				)
				d.ShowModal()
		except Exception as e:
			logging.error(traceback.format_exc())
			if force:
				d = wx.MessageDialog(
					parent,
					_("There was an error while checking for updates.\nPlease try again later."),
					_("Update error"),
					wx.OK | wx.ICON_ERROR
				)
				d.ShowModal()

	delayedresult.startWorker(consume_updates, check_for_updates)


class MyUpdateDialog(UpdateDialog):
	def __init__(self, parent, preferences, updates):
		UpdateDialog.__init__(self, parent)
		self.preferences = preferences
		self.updates = updates
		self.updatePanels = set()
		for u in updates:
			up = MyUpdatePanel(self, preferences, u[0], u[1], u[2])
			self.updatePanels.add(up)
			self.updatesSizer.Add(up, 1, wx.EXPAND, 5 )
		self.SetSize((400, 400))
		
	def RemoveChild(self, child):
		self.updatesSizer.Layout()
		self.updatePanels.remove(child)
		if len(self.updatePanels) == 0:
			self.Show(False)
			self.Destroy()
			
	def OnDonate(self, evt):
		wx.LaunchDefaultBrowser(_("https://www.skeed.it/songpress#donate"))
		evt.Skip()




# -*- coding: iso-8859-1 -*-

###############################################################
# Name:			 EditDistance.py
# Purpose:	 Minimum Edit Distance algorithm
# Code from: http://www.cs.colorado.edu/~martin/csci5832/edit-dist-blurb.html
# Created:	 2013-08-14
##############################################################


def substCost(x,y):
	if x == y: return 0
	else: return 1
	
def insertCost(x):
	return 1

def deleteCost(x):
	return 1

def  minEditDist(target, source):
	n = len(target)
	m = len(source)

	distance = [[0 for i in range(m+1)] for j in range(n+1)]

	for i in range(1,n+1):
		distance[i][0] = distance[i-1][0] + insertCost(target[i-1])

	for j in range(1,m+1):
		distance[0][j] = distance[0][j-1] + deleteCost(source[j-1])

	for i in range(1,n+1):
		for j in range(1,m+1):
		   distance[i][j] = min(
					distance[i-1][j]+1,
					distance[i][j-1]+1,
					distance[i-1][j-1]+substCost(source[j-1],target[i-1])
				)
	return distance[n][m]



###############################################################
# Name:			 SongFormat.py
# Purpose:	 Song format options
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2009-02-24
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

import wx


class AttributeMonitor(object):
	def __init__(self, handlers):
		object.__init__(self)
		self.am_started = False
		self.am_handlers = handlers

	def __setattr__(self, name, value):
		object.__setattr__(self, name, value)
		if name != 'am_started' and self.am_started:
			for h in self.am_handlers:
				if name in h.names:
					#print name
					h.OnSetAttr(self, name, value)

	def Am_Start(self):
		self.am_started = True


class FontChangeHandler(object):
	names = set(("face", "size", "bold", "italic", "underline"))
	@staticmethod
	def OnSetAttr(format, name, value):
		format.UpdateWxFont()


class FontFormat(AttributeMonitor):
	def __init__(self, ff=None):
		AttributeMonitor.__init__(self, (FontChangeHandler, ))
		# Definable by user
		if ff == None:
			self.face = "Arial"
		else:
			self.face = ff.face
		self.size = 12 if ff is None else ff.size
		self.bold = False if ff is None else ff.bold
		self.italic = False if ff is None else ff.italic
		self.underline = False  if ff is None else ff.underline
		self.color = '#000000' if ff is None else ff.color
		# Auto set
		self.wxFont = None
		self.UpdateWxFont()
		# Start monitor
		self.Am_Start()

	def UpdateWxFont(self):
		#print("Font Updated, face = " + self.face)
		if self.italic:
			style = wx.FONTSTYLE_ITALIC
		else:
			style = wx.FONTSTYLE_NORMAL
		if self.bold:
			weight = wx.FONTWEIGHT_BOLD
		else:
			weight = wx.FONTWEIGHT_NORMAL
		self.wxFont = wx.Font(int(self.size), wx.FONTFAMILY_DEFAULT, style, weight, self.underline, self.face)


class ParagraphFormat(FontFormat):
	def __init__(self, ff=None):
		FontFormat.__init__(self, ff)
		self.leftMargin = 0 if ff is None else ff.leftMargin
		self.topMargin = 12 if ff is None else ff.topMargin
		self.bottomMargin = 0 if ff is None else ff.bottomMargin
		self.chordSpacing = 0.8 if ff is None else ff.chordSpacing
		self.textSpacing = 1 if ff is None else ff.textSpacing
		self.chord = FontFormat(ff.chord) if ff is not None else FontFormat()
		self.comment = FontFormat(ff.comment) if ff is not None else FontFormat()
		if ff is None:
			self.chord.size = self.size * 0.9
			self.chord.italic = True
			self.chord.bold = False
			self.comment.italic = True


class SongFormat(ParagraphFormat):
	def __init__(self, ff=None):
		ParagraphFormat.__init__(self, ff)
		self.verse = []
		self.chorus = ParagraphFormat(ff.chorus) if ff is not None else ParagraphFormat()
		self.title = ParagraphFormat(ff.title) if ff is not None else ParagraphFormat()
		self.subtitle = ParagraphFormat(ff.subtitle) if ff is not None else ParagraphFormat()
		self.blockSpacing = 1 if ff is None else ff.blockSpacing
		# showChords:
		# 0. None
		# 1. First verse and chorus
		# 2. Entire song
		self.showChords = 1 if ff is None else ff.showChords
		if ff is None:
			self.chorus.bold = True
			self.chorus.underline = False
			self.title.bold = True
			self.title.underline = True
			self.subtitle.size = self.title.size * 0.95
			self.subtitle.italic = True

	def AddVerse(self):
		self.verse.append(ParagraphFormat(self))

	def InitVerses(self):
		self.verse = []


###############################################################
# Name:			 MyDecoSlider.py
# Purpose:	 Slider with a decorated range indicator
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2010-06-02
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

import wx

from .DecoSlider import *

class MyDecoSlider(DecoSlider):
	def __init__(self, parent):
		DecoSlider.__init__(self, parent)

	def OnPaint(self, event):
		dc = wx.PaintDC(self.panel)
		w, h = dc.GetSize()
		dc.Clear()
		green =	wx.Colour(0, 255, 0)
		brush = wx.Brush(green, wx.SOLID)
		dc.SetBrush(brush)
		dc.DrawPolygon([wx.Point(0, 0), wx.Point(w - 1, 0), wx.Point(w - 1, h - 1)])

	def OnSize(self, event):
		self.panel.Refresh()



"""Framework providing common functionality for SDI Main Frames

The module provides an (abstract) class loads an XRC resource describing a SDI Frame.
It expects to find some menu elements, characterized by their XRC name:
	- new
	- open
	- save
	- saveAs
	- exit
	- about
"""

###############################################################
# Name:			 SDIMainFrame.py
# Purpose:	 Abstract class for SDI Main Frames
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2009-01-16
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

import os
import sys
import platform

import wx
# import wx.aui as aui
from wx.lib.agw import aui
from wx import xrc

from . import i18n


i18n.register('songpress.SDIMainFrame')

class SDIDropTarget(wx.FileDropTarget):
	def __init__(self, sdi):
		wx.FileDropTarget.__init__(self)
		self.sdi = sdi

	def OnDropFiles(self, x, y, arr):
		return self.sdi.OnDropFiles(arr)


class SDIMainFrame(wx.FileDropTarget):
	"""Abstract class. Override methods New, Open, Save"""
	###UI generation###

	def __init__(
		self,
		res,
		frameName='MainFrame',
		appName='SDIApp',
		authorName='Nobody',
		docType='document',
		docExt='txt',
		appLongName=None,
		icon=None,
		version="1.0",
		url="",
		copyright = "",
		licensing = "",
		thanks = "",
		importFormats = [] # List of tuples: (format name, [extensions])
	):
		self.config = wx.Config.Get()
		self.res = res
		i18n.localizeXrc('xrc/songpress.xrc')
		self.appName = appName
		self.appLongName = self.appName if appLongName == None else appLongName
		self.authorName = authorName
		self.modified = False
		self.version = version
		self.url = url
		self.copyright = copyright
		self.licensing = licensing
		self.thanks = thanks
		self.document = ''
		self.docType = docType
		self.docExt = docExt
		self.importFormats = importFormats
		wx.Config.Set(self.config)
		self.frame = self.res.LoadFrame(None, frameName)
		if icon != None:
			self.frame.SetIcon(wx.Icon(icon, wx.BITMAP_TYPE_ICO))
		self.BindMenu()
		self.frame.Bind(wx.EVT_CLOSE, self.OnClose, self.frame)
		dt = SDIDropTarget(self)
		self.frame.SetDropTarget(dt)
		self.UpdateTitle()
		self._mgr = aui.AuiManager(self.frame, aui.AUI_MGR_ALLOW_FLOATING | aui.AUI_MGR_LIVE_RESIZE | aui.AUI_MGR_HINT_FADE)
		self._mgr.Bind(aui.EVT_AUI_PANE_CLOSE, self.OnPaneClose)
		self.menuBar = self.frame.GetMenuBar()
		self.panesByMenu = {}
		self.menusByPane = {}
		self.RetrieveRecentFileList()

	def Bind(self, event, handler, xrcname):
		"""Bind an event, coming from a control by xrc name, to a handler"""
		self.frame.Bind(event, handler, id=xrc.XRCID(xrcname))

	def BindMenu(self):
		"""Bind a menu item, by xrc name, to a handler"""
		def Bind(handler, xrcname):
			self.Bind(wx.EVT_MENU, handler, xrcname)

		Bind(self.OnNew, 'new')
		Bind(self.OnOpen, 'open')
		Bind(self.OnSave, 'save')
		Bind(self.OnSaveAs, 'saveAs')
		Bind(self.OnExit, 'exit')
		Bind(self.OnAbout, 'about')
		
	def SetDefaultExtension(self, ext):
		self.docExt = ext

	def OnNew(self, evt):
		"""Menu handler for File->New"""
		if self.AskSaveModified():
			self.document = ''
			self.New()
			self.modified = False
			self.UpdateTitle()

	def OnOpen(self, evt):
		"""Menu handler for File->Open"""
		if self.AskSaveModified():
			if self.importFormats == []:
				filter = _("%s files (*.%s)|%s") % (
						self.docExt,
						self.docExt,
						self.docExt,
					)
			else:
				filter = "|".join(["%s|%s" % (x[0], ";".join(["*." + y for y in x[1]])) for x in self.importFormats])
			filter += _("|All files (*.*)|*.*")
			dlg = wx.FileDialog(
				self.frame,
				_("Open file"),
				"",
				"",
				filter,
				wx.FD_OPEN
			)

			if dlg.ShowModal() == wx.ID_OK:
				fn = dlg.GetPath()
				if os.path.isfile(fn):
					self.document = fn
					self.Open()
					self.UpdateRecentFileList(fn)
					self.modified = False
					self.UpdateTitle()
				else:
					msg = _("File \"%s\" does not exist.") % (fn, )
					d = wx.MessageDialog(
						self.frame,
						msg,
						self.appLongName,
						wx.OK | wx.ICON_ERROR
					)
					d.ShowModal()

	def OnSave(self, evt):
		"""Menu handler for File->Save"""
		self.SaveFile()

	def OnSaveAs(self, evt):
		"""Menu handler for File->Save As"""
		if self.AskSaveFilename():
			self.SaveFile()

	def OnExit(self, evt):
		"""Menu handler for File->Exit"""
		self.frame.Close()

	def OnAbout(self, evt):
		"""Menu handler for ?->About"""
		msg = _("%s version %s\n%s\n\n%s\n\n%s\n\n%s") % (
			self.appLongName,
			self.version,
			self.url,
			self.copyright,
			self.licensing,
			self.thanks
		)
		wx.MessageBox(msg, _('About ') + self.appLongName)

	def OnDropFiles(self, arr):
		"""Handler for drop action: opens the dropped file, if it is exactly one"""
		if len(arr) == 1:
			fn = arr[0]
			if os.path.isfile(fn) and self.AskSaveModified():
				self.document = fn
				self.Open()
				self.UpdateRecentFileList(fn)
				self.modified = False
				self.UpdateTitle()
				return True
		return False

	def OnClose(self, evt):
		"""Handler for windows close event"""
		if self.AskSaveModified(evt.CanVeto()):
			self.config.SetPath('/SDIMainFrame')
			self.config.Write("Version", self.version)
			p = self._mgr.SavePerspective()
			self.config.Write("Perspective", p)
			self.SavePreferences()
			self._mgr.UnInit()
			del self._mgr
			self.frame.Destroy()
		else:
			evt.Veto()

	###Ordinary methods###

	def SetModified(self, m = True):
		"""Set the modified flag, if main document is modified"""
		self.modified = m
		self.UpdateTitle()

	def UpdateTitle(self):
		"""Updates form title; to be called when the filename or the modified status changes"""
		if self.modified:
			mod = '* '
		else:
			mod = ''
		if self.document == '':
			doc = _('Untitled')
		else:
			doc = os.path.basename(self.document)
			(doc, ext) = os.path.splitext(doc)
		self.frame.SetTitle(u"%s%s - %s" % (mod, doc, self.appLongName))

	def AskSaveModified(self, canCancel = True):
		"""If file has been modified, propose to save changes. Return False if cancelled, True otherwise"""
		if not self.modified:
			return True

		if canCancel:
			cc = wx.CANCEL
		else:
			cc = 0

		d = wx.MessageDialog(self.frame, _("Your %s has been modified. Do you want to save it?") % (self.docType), self.appLongName, wx.YES_NO | wx.ICON_QUESTION | cc)
		res = d.ShowModal()
		if res == wx.ID_CANCEL:
			return False
		elif res == wx.ID_NO:
			return True
		else: #wxID_YES
			return self.SaveFile();

	def AskSaveFilename(self):
		"""Ask and updates the filename (without saving); return False if user cancels, True otherwise"""
		leave = False;
		consensus = False;
		while not leave:
			dlg = wx.FileDialog(
				self.frame,
				_("Choose a name for the file"),
				"",
				"",
				_("%s files (*.%s)|*.%s|All files (*.*)|*.*") % (
						self.docExt,
						self.docExt,
						self.docExt,
					),
				wx.FD_SAVE
			)

			if dlg.ShowModal() == wx.ID_OK:

				fn = dlg.GetPath()
				if os.path.isfile(fn):
					msg = _("File \"%s\" already exists. Do you want to overwrite it?") % (fn, )
					d = wx.MessageDialog(
						self.frame,
						msg,
						self.appLongName,
						wx.YES_NO | wx.CANCEL | wx.ICON_QUESTION
					)
					res = d.ShowModal()
					if res == wx.ID_CANCEL:
						leave = True
						consensus = False
					elif res == wx.ID_NO:
						leave = False
						consensus = False
					else: #wxID_YES
						leave = True
						consensus = True
				else:
					leave = True
					consensus = True

			else:
				leave = True
				consensus = False

		if consensus:
			if platform.system() == 'Linux':
				# Since wxPython file dialog in Linux does not add default extension
				# when it has not been specified by user, we add it ourselves
				pref, ext = os.path.splitext(fn)
				if ext == '':
					fn = '%s.%s' % (fn, self.docExt)
			self.document = fn
			self.UpdateTitle()
			return wx.ID_OK
		else:
			return False

	def SaveFile(self):
		"""Save file, asking for file name if necessary. Return False if user cancels, True otherwise"""
		if self.document == '':
			if not self.AskSaveFilename():
				return False
		self.Save()
		self.UpdateRecentFileList(self.document)
		self.modified = False
		self.UpdateTitle()
		return True

	def New(self):
		"""To be overridden: create a blank document"""
		pass

	def Open(self):
		"""To be overridden: open a document"""
		pass

	def Save(self):
		"""To be overridden: save a document"""
		return True

	def SavePreferences(self):
		"""To be overridden: save preferences to wxConfig object"""
		pass

	def AddMainPane(self, window):
		self._mgr.AddPane(window, aui.AuiPaneInfo().CenterPane().Name('_main'))

	def AddPane(self, window, info, caption, menuName):
		self._mgr.AddPane(window, info.Name(menuName).Caption(caption))
		menuid = xrc.XRCID(menuName)
		self.panesByMenu[menuid] = menuName
		self.menusByPane[menuName] = menuid
		self.Bind(wx.EVT_MENU, self.OnTogglePaneView, menuName)

	def OnTogglePaneView(self, evt):
		status = evt.GetInt()
		menu = evt.GetId()
		pane = self._mgr.GetPane(self.panesByMenu[menu])
		pane.Show(status)
		self._mgr.Update()

	def OnPaneClose(self, evt):
		pane = evt.GetPane()
		menuid = self.menusByPane[pane.name]
		self.menuBar.Check(menuid, False)

	def RetrieveRecentFileList(self):
		self.recentMenuBase = 800
		self.config.SetPath('/SDIMainFrame/RecentFiles')
		self.recentFiles = []
		for i in range(1, 5):
			f = self.config.Read(str(i))
			if f:
				self.recentFiles.append(f)
			self.frame.Bind(wx.EVT_MENU, self.OnRecentFile, id = self.recentMenuBase + i)
		self.UpdateRecentFileMenu()

	def EmptyRecentFileMenu(self):
		fileMenu = self.menuBar.GetMenu(0)
		for i in range(1, len(self.recentFiles)+1):
			fileMenu.Remove(self.recentMenuBase + i)

	def UpdateRecentFileMenu(self):
		i = 1
		fileMenu = self.menuBar.GetMenu(0)
		self.config.SetPath('/SDIMainFrame/RecentFiles')
		for k in self.recentFiles:
			d, f = os.path.split(k)
			if len(d) > 25:
				d = d[:25] + "..."
			fileMenu.Append(self.recentMenuBase + i, os.path.join(d, f))
			self.config.Write(str(i), k)
			i += 1

	def UpdateRecentFileList(self, name):
		self.EmptyRecentFileMenu()
		if name in self.recentFiles:
			self.recentFiles.remove(name)
		self.recentFiles.insert(0, name)
		if len(self.recentFiles) > 4:
			self.recentFiles.pop()
		self.UpdateRecentFileMenu()

	def OnRecentFile(self, evt):
		i = evt.GetId() - self.recentMenuBase
		fn = self.recentFiles[i-1]
		if os.path.isfile(fn) and self.AskSaveModified():
			self.document = fn
			self.Open()
			self.UpdateRecentFileList(fn)
			self.modified = False
			self.UpdateTitle()

	def FinalizePaneInitialization(self):
		self.config.SetPath('/SDIMainFrame')
		v = self.config.Read("Version", "0.0")
		vs = v.split('.')
		svs = self.version.split('.')
		p = False
		if vs[:2] == svs[:2]:
			p = self.config.Read("Perspective")
		#print "Config: " + str(p)
		if p:
			self._mgr.LoadPerspective(p)
			for menuid in self.panesByMenu:
				self.menuBar.Check(menuid, self._mgr.GetPane(self.panesByMenu[menuid]).IsShown())
		else:
			self._mgr.Update()
		if len(sys.argv) > 1:
			self.OnDropFiles([sys.argv[1]])
		self.frame.Show()



###############################################################
# Name:			 Tokenizer.py
# Purpose:	 Abstract tokenizer
# Author:		 Luca Allulli (webmaster@roma21.it)
# Created:	 2009-01-31
# Copyright: Luca Allulli (https://www.skeed.it/songpress)
# License:	 GNU GPL v2
##############################################################

import re


class Token(object):
	def __init__(self, token, start, end, content):
		object.__init__(self)
		self.token = token
		self.start = start
		self.end = end
		self.content = content

	def __str__(self):
		return '%s, %d, %d, %s' % (str(self.token), self.start, self.end, self.content)


class TokenType(object):
	def __init__(self, r, name):
		object.__init__(self)
		self.r = re.compile(r)
		self.name = name

	def __str__(self):
		return self.name


class Tokenizer(object):
	def __init__(self, l):
		object.__init__(self)
		self.line = l
		self.pos = 0
		self.state = self.normal
		self.prevToken = None
		self.repeatToken = False

	def __iter__(self):
		return self

	def next(self):
		return self.__next__()

	def __next__(self):
		if self.repeatToken:
			self.repeatToken = False
			return self.prevToken
		if self.pos >= len(self.line):
			#End of line
			raise StopIteration
		for t in self.transition[self.state]:
			# print("Trying: %s" % (t[0],))
			m = t[0].r.match(self.line, self.pos)
			if m != None:
				# print("Matched!")
				break
		if m == None:
			return None
		tok = Token(t[0], self.pos, m.end(0), m.group(0))
		##print (type(m.group(0)), m.group(0))
		self.state = t[1]
		##print("Found %s" % (tok,))
		self.pos = m.end(0)
		self.prevToken = tok
		return tok

	def Repeat(self, repeat = True):
		self.repeatToken = repeat



# -*- coding: utf-8 -*- 

from . import i18n

i18n.register('songpress.UpdateDialog')

# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Sep  8 2010)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx

###########################################################################
## Class UpdateDialog
###########################################################################

class UpdateDialog ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = _("New updates are available"), pos = wx.DefaultPosition, size = wx.Size( 258,250 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		bSizer20 = wx.BoxSizer( wx.VERTICAL )
		
		self.updatesSizer = wx.BoxSizer(wx.VERTICAL)
		
		bSizer20.Add( self.updatesSizer, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.donate = wx.Button( self, wx.ID_ANY, _(">> Donate now! <<"), wx.DefaultPosition, wx.DefaultSize, 0 )
		bSizer20.Add( self.donate, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.SetSizer( bSizer20 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.donate.Bind( wx.EVT_BUTTON, self.OnDonate )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnDonate( self, event ):
		event.Skip()
	

